'use strict';

var fs = require('node:fs/promises');
var path = require('node:path');
var fastXmlParser = require('fast-xml-parser');
var fs$1 = require('node:fs');
var node_crypto = require('node:crypto');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var fs__namespace = /*#__PURE__*/_interopNamespaceDefault(fs);
var path__namespace = /*#__PURE__*/_interopNamespaceDefault(path);
var fs__namespace$1 = /*#__PURE__*/_interopNamespaceDefault(fs$1);

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

const XML_HEADER = '<?xml version="1.0" encoding="UTF-8"?>';
const INDENT = "    ";

const XML_PARSER_OPTION = {
    commentPropName: "!---",
    ignoreAttributes: false,
    ignoreNameSpace: false,
    parseTagValue: false,
    parseNodeValue: false,
    parseAttributeValue: false,
    trimValues: true,
    processEntities: false,
    cdataPropName: "![CDATA[",
};
const JSON_PARSER_OPTION = Object.assign(Object.assign({}, XML_PARSER_OPTION), { format: true, indentBy: INDENT, suppressBooleanAttributes: false, suppressEmptyNode: false });

function buildReassembledFile(combinedXmlContents, filePath, xmlElement, xmlNamespace) {
    return __awaiter(this, void 0, void 0, function* () {
        let finalXmlContent = combinedXmlContents.join("\n");
        finalXmlContent = finalXmlContent.replace(/<\?xml version="1.0" encoding="UTF-8"\?>/g, "");
        finalXmlContent = finalXmlContent.replace(new RegExp(`<${xmlElement}[^>]*${xmlNamespace ? `( xmlns="${xmlNamespace}")?` : ""}>`, "g"), "");
        finalXmlContent = finalXmlContent.replace(new RegExp(`</${xmlElement}>`, "g"), "");
        finalXmlContent = finalXmlContent.replace(/<!\[CDATA\[\s*([\s\S]*?)\s*]]>/g, (match, cdataContent) => {
            const trimmedContent = cdataContent.trim();
            const lines = trimmedContent.split("\n");
            const indentedLines = lines.map((line) => line.replace(/^\s*/, ""));
            return `<![CDATA[\n${INDENT}${indentedLines.join(`\n${INDENT}`)}\n]]>`;
        });
        finalXmlContent = finalXmlContent.replace(/(\n\s*){2,}/g, `\n${INDENT}`);
        const openTag = xmlNamespace
            ? `<${xmlElement} xmlns="${xmlNamespace}">`
            : `<${xmlElement}>`;
        const closeTag = `</${xmlElement}>`;
        yield fs__namespace.writeFile(filePath, `${XML_HEADER}\n${openTag}${finalXmlContent}${closeTag}`);
        console.log(`Created reassembled file: ${filePath}`);
    });
}

class ReassembleXMLFileHandler {
    processFilesInDirectory(dirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const combinedXmlContents = [];
            const files = yield fs__namespace.readdir(dirPath);
            files.sort((fileA, fileB) => {
                const fullNameA = fileA.split(".")[0].toLowerCase();
                const fullNameB = fileB.split(".")[0].toLowerCase();
                return fullNameA.localeCompare(fullNameB);
            });
            for (const file of files) {
                const filePath = path__namespace.join(dirPath, file);
                const fileStat = yield fs__namespace.stat(filePath);
                if (fileStat.isFile()) {
                    const xmlContent = yield fs__namespace.readFile(filePath, "utf-8");
                    combinedXmlContents.push(xmlContent);
                }
                else if (fileStat.isDirectory()) {
                    const subdirectoryContents = yield this.processFilesInDirectory(filePath);
                    combinedXmlContents.push(...subdirectoryContents);
                }
            }
            return combinedXmlContents;
        });
    }
    processFilesForRootElement(dirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield fs__namespace.readdir(dirPath);
            const xmlParser = new fastXmlParser.XMLParser(XML_PARSER_OPTION);
            for (const file of files) {
                const filePath = path__namespace.join(dirPath, file);
                const fileStat = yield fs__namespace.stat(filePath);
                if (fileStat.isDirectory()) {
                    const result = yield this.processFilesForRootElement(filePath);
                    if (result !== undefined) {
                        return result;
                    }
                }
                else if (fileStat.isFile()) {
                    const xmlContent = yield fs__namespace.readFile(filePath, "utf-8");
                    const xmlParsed = xmlParser.parse(xmlContent);
                    const rootElementName = Object.keys(xmlParsed)[1];
                    if (rootElementName !== undefined) {
                        return rootElementName;
                    }
                }
            }
            return undefined;
        });
    }
    reassemble(xmlAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xmlPath, xmlNamespace, fileExtension } = xmlAttributes;
            const combinedXmlContents = [];
            const fileStat = yield fs__namespace.stat(xmlPath);
            if (!fileStat.isDirectory()) {
                throw new Error("The provided xmlPath is not a directory.");
            }
            const filesInxmlPath = yield fs__namespace.readdir(xmlPath);
            for (const file of filesInxmlPath) {
                const filePath = path__namespace.join(xmlPath, file);
                const fileStat = yield fs__namespace.stat(filePath);
                if (fileStat.isFile()) {
                    const xmlContent = yield fs__namespace.readFile(filePath, "utf-8");
                    combinedXmlContents.push(xmlContent);
                }
                else if (fileStat.isDirectory()) {
                    const subdirectoryContents = yield this.processFilesInDirectory(filePath);
                    combinedXmlContents.push(...subdirectoryContents);
                }
            }
            let rootElementName = yield this.processFilesForRootElement(xmlPath);
            const parentDirectory = path__namespace.dirname(xmlPath);
            const subdirectoryBasename = path__namespace.basename(xmlPath);
            const fileName = fileExtension
                ? `${subdirectoryBasename}.${fileExtension}`
                : `${subdirectoryBasename}.xml`;
            const filePath = path__namespace.join(parentDirectory, fileName);
            if (rootElementName !== undefined) {
                yield buildReassembledFile(combinedXmlContents, filePath, rootElementName, xmlNamespace);
            }
            else {
                console.error("Root element name is undefined");
            }
        });
    }
}

function findUniqueIdElement(element, uniqueIdElements) {
    if (uniqueIdElements === undefined) {
        return getShortHash(element);
    }
    const uniqueIdElementsArray = uniqueIdElements.split(",");
    for (const fieldName of uniqueIdElementsArray) {
        if (element[fieldName] !== undefined) {
            if (typeof element[fieldName] === "string") {
                return element[fieldName];
            }
        }
    }
    for (const key in element) {
        if (typeof element[key] === "object" && element[key] !== null) {
            const childFieldName = findUniqueIdElement(element[key], uniqueIdElements);
            if (childFieldName !== undefined) {
                return childFieldName;
            }
        }
    }
    return getShortHash(element);
}
function getShortHash(element) {
    const hash = node_crypto.createHash("sha256");
    hash.update(JSON.stringify(element));
    const fullHash = hash.digest("hex");
    return fullHash.slice(0, 8);
}

function buildNestedElements(element, indentLevel = 2) {
    const xmlBuilder = new fastXmlParser.XMLBuilder(JSON_PARSER_OPTION);
    const xmlString = xmlBuilder.build(element);
    const formattedXml = xmlString
        .split("\n")
        .map((line) => `${" ".repeat(indentLevel * INDENT.length)}${line}`)
        .join("\n")
        .trimEnd();
    return formattedXml;
}

function buildDisassembledFiles(xmlString, metadataPath, uniqueIdElements, baseName, indent) {
    const xmlParser = new fastXmlParser.XMLParser(XML_PARSER_OPTION);
    const result = xmlParser.parse(xmlString);
    const rootElementName = Object.keys(result)[1];
    const rootElement = result[rootElementName];
    let leafContent = "";
    let leafCount = 0;
    Object.keys(rootElement)
        .filter((key) => !key.startsWith("@"))
        .forEach((key) => {
        if (Array.isArray(rootElement[key])) {
            for (const element of rootElement[key]) {
                if (typeof element === "object") {
                    buildNestedFile(element, metadataPath, uniqueIdElements, rootElementName, key, indent);
                }
                else {
                    const fieldValue = element;
                    leafContent += `${indent}<${key}>${String(fieldValue)}</${key}>\n`;
                    leafCount++;
                }
            }
        }
        else if (typeof rootElement[key] === "object") {
            buildNestedFile(rootElement[key], metadataPath, uniqueIdElements, rootElementName, key, indent);
        }
        else {
            const fieldValue = rootElement[key];
            leafContent += `${indent}<${key}>${String(fieldValue)}</${key}>\n`;
            leafCount++;
        }
    });
    if (leafCount > 0) {
        let leafFile = `${XML_HEADER}\n`;
        leafFile += `<${rootElementName}>\n`;
        const sortedLeafContent = leafContent
            .split("\n")
            .filter((line) => line.trim() !== "")
            .sort()
            .join("\n");
        leafFile += sortedLeafContent;
        leafFile += `\n</${rootElementName}>`;
        const leafOutputPath = path__namespace.join(metadataPath, `${baseName}.xml`);
        fs__namespace$1.writeFileSync(leafOutputPath, leafFile);
        console.log(`All leaf elements saved to: ${leafOutputPath}`);
    }
}
function buildNestedFile(element, metadataPath, uniqueIdElements, rootElementName, parentKey, indent) {
    let elementContent = "";
    const fieldName = findUniqueIdElement(element, uniqueIdElements);
    const outputDirectory = path__namespace.join(metadataPath, parentKey);
    const outputFileName = `${fieldName}.${parentKey}-meta.xml`;
    const outputPath = path__namespace.join(outputDirectory, outputFileName);
    fs__namespace$1.mkdirSync(outputDirectory, { recursive: true });
    elementContent = buildNestedElements(element);
    let decomposeFileContents = `${XML_HEADER}\n`;
    decomposeFileContents += `<${rootElementName}>\n`;
    decomposeFileContents += `${indent}<${parentKey}>\n`;
    decomposeFileContents += `${elementContent}\n`;
    decomposeFileContents += `${indent}</${parentKey}>\n`;
    decomposeFileContents += `</${rootElementName}>`;
    fs__namespace$1.writeFileSync(outputPath, decomposeFileContents);
    console.log(`XML content saved to: ${outputPath}`);
}

class DisassembleXMLFileHandler {
    disassemble(xmlAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xmlPath, uniqueIdElements } = xmlAttributes;
            const fileStat = yield fs__namespace.stat(xmlPath);
            if (!fileStat.isDirectory()) {
                throw new Error("The provided xmlPath is not a directory.");
            }
            const files = yield fs__namespace.readdir(xmlPath);
            for (const file of files) {
                const filePath = path__namespace.join(xmlPath, file);
                yield this.processFile({
                    xmlPath,
                    filePath,
                    uniqueIdElements,
                });
            }
        });
    }
    processFile(xmlAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xmlPath, filePath, uniqueIdElements } = xmlAttributes;
            if (filePath.endsWith(".xml")) {
                console.log(`Parsing file: ${filePath}`);
                const xmlContent = yield fs__namespace.readFile(filePath, "utf-8");
                const fullName = path__namespace.basename(filePath, path__namespace.extname(filePath));
                const baseName = fullName.split(".")[0];
                let outputPath;
                outputPath = path__namespace.join(xmlPath, baseName);
                buildDisassembledFiles(xmlContent, outputPath, uniqueIdElements, fullName, INDENT);
            }
        });
    }
}

exports.DisassembleXMLFileHandler = DisassembleXMLFileHandler;
exports.ReassembleXMLFileHandler = ReassembleXMLFileHandler;
//# sourceMappingURL=index.cjs.map
